"""
Personal Kanban - Personal organizer
Copyright (C) 2017-2018 Lorenzo Ancora

This file is part of Personal Kanban.

Personal Kanban is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Personal Kanban is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import os
import sys
import time
import random
import logging
from signal import signal, SIGINT, SIG_IGN
from datetime import datetime
from termenu import Termenu, ansi
from PersonalKanban import PersonalKanban

version = 1.0
reqexit = False
logger = logging.getLogger()

def draw_warning(title: str, text: str):
    maxlinelen = 0
    for line in text.splitlines():
        curlinelen = len(line)
        if(maxlinelen < curlinelen):
            maxlinelen = curlinelen
    ansi.save_position()
    ansi.hide_cursor()
    ansi.clear_line()
    print("")
    ansi.move_horizontal(3)
    sys.stdout.write("╭── " + title + " " + '─'*(
        maxlinelen-len(title)-2) + "╮\n")
    ansi.move_horizontal(3)
    sys.stdout.write("│ ❍" + ' '*(max(len(title)+2, maxlinelen+2)-4) + "❍ │\n")
    sys.stdout.flush()
    for textline in text.splitlines():
        ansi.clear_line()
        ansi.move_horizontal(3)
        sys.stdout.write("╰" + '─'*max(len(title)+2, maxlinelen+2) + "╯")
        sys.stdout.flush()
        time.sleep(0.1)
        ansi.move_horizontal(3)
        sys.stdout.write("│ {noticel}{spaces} │\n".
                         format(noticel=textline, spaces=' '*(
                             maxlinelen-len(textline))))
        sys.stdout.flush()
        time.sleep(0.2)
    ansi.clear_line()
    ansi.move_horizontal(3)
    sys.stdout.write("│ ❍" + ' '*(max(len(title)+2, maxlinelen+2)-4) + "❍ │\n")
    ansi.move_horizontal(3)
    sys.stdout.write("╰" + '─'*max(len(title)+2, maxlinelen+2) + "╯")
    sys.stdout.flush()
    time.sleep(0.2)
    ansi.restore_position()
    ansi.show_cursor()
    time.sleep(0.5)

    return


kanban = PersonalKanban()

storedir = os.path.expanduser("~/.kanban")
storefile = storedir + "/personal.dat"
if not os.path.isdir(storedir):
    os.mkdir(storedir)
if not os.path.isfile(storefile):
    kanban.save(storefile)
else:
    kanban.load(storefile)


def clear_screen():
    ansi.clear_screen()
    ansi.move(2, 2)

    return


def draw_kanban():
    ansi.hide_cursor()
    todo_n = len(kanban.tasks["todo"])
    for depth in range(0, 3 - todo_n):
        ansi.move_horizontal(13)
        if(todo_n == 0):
            print(ansi.colorize("┃", "red"), end='')
            print(ansi.colorize("░░░░░", "black"), end='')
            print(ansi.colorize("┃", "red"))
        else:
            print("┃", end='')
            print(ansi.colorize(" ░░░ ", "black"), end='')
            print("┃")
    cntcolors = ["green", "white", "red"]
    for depth, task in enumerate(kanban.tasks["todo"]):
        ansi.move_horizontal(13)
        curcnt = ansi.colorize("▒▓█▓▒", cntcolors[depth])
        print("┃{cnt}\033[4m┃ {desc} \033[0m".format(
            cnt=curcnt, desc=task[:30]))
    ansi.move_horizontal(13)
    boxbottom = "┗━━━━━┛"
    if(todo_n == 0):
        print(ansi.colorize(boxbottom, "red"))
    else:
        print(boxbottom)
    ansi.move_horizontal(15)
    print(str(todo_n) + "/3")
    ansi.restore_position()
    ansi.show_cursor()
    sys.stdout.flush()

    return


def exit_dialog(signum, stackframe):
    signal(SIGINT, SIG_IGN)
    clear_screen()
    draw_warning(title="Forced quit [CTRL+C]",
                 text="The latest edits were not saved.\nRestart the program.")
    for remt in range(3, 0, -1):
        print(remt, end='')
        ansi.restore_position()
        sys.stdout.flush()
        time.sleep(1.0)
        ansi.clear_line()
        sys.stdout.flush()
    exit(0)


def init_screen():
    clear_screen()
    random.seed(a=time.clock())
    colnum = 70
    simnum = int(colnum / 4)
    for r in range(12):
        ansi.move_horizontal(6)
        ch = random.choice((True, False))
        astr = "▉▞▚▉"
        bstr = ''.join(random.sample(astr, len(astr)))
        if ch is True:
            sys.stdout.write(astr*simnum)
        else:
            sys.stdout.write(bstr*simnum)
        sys.stdout.write("\n")
    ansi.move(6, 19)
    sys.stdout.write("Personal Kanban")
    ansi.move(7, 25)
    sys.stdout.write("ver. {SVER} (soft), {CVER} (class)".format(
        SVER=version,
        CVER=kanban.version))
    ansi.move(11, 28)
    sys.stdout.write("Copyright (C) 2017-2018 Lorenzo L. Ancora")
    ansi.move(12, 40)
    sys.stdout.write("License GPLv3")
    sys.stdout.flush()

    return


for i in range(7):
    init_screen()
    time.sleep(0.3)
signal(SIGINT, exit_dialog)


def prepare_item(item: str) -> str:
    if(item[-1] == "."):
        item = item[:-1]
    if(item[0].islower()):
        item = item.capitalize()

    return item


def subMenuList():
    print("""Use SPACE for multiple selection.
    Press ENTER to browse the lists.""")
    submenu = Termenu(["Todo", "Options", "Completed"], multiselect=True)

    qresult = submenu.show()
    if("Options" in qresult):
        glststr = "OPTIONS: [{cn}]".format(cn=len(kanban.tasks["options"]))
        for option in kanban.tasks["options"]:
            option = prepare_item(option)
            glststr += "\n❖ {o};".format(o=option)
        print(glststr)
        input()
    if("Todo" in qresult):
        glststr = "TODOs: [{cn}/3]".format(cn=len(kanban.tasks["todo"]))
        for task in kanban.tasks["todo"]:
            task = prepare_item(task)
            glststr += "\n❖ {o};".format(o=task)
        print(glststr)
        input()
    if("Completed" in qresult):
        glststr = "COMPLETED TASKS: [{cn}]".format(
            cn=len(kanban.tasks["done"]))
        for task in kanban.tasks["done"]:
            localtime = datetime.fromtimestamp(
                int(task[1])).strftime('%d/%m/%Y %H:%M')
            task = prepare_item(task)
            glststr += "\n❖ {tn}\t\tat {ct}".format(
                tn=task[0], ct=localtime)
        if(len(kanban.tasks["done"]) > 0):
            glststr += "\n--- WELL DONE! ---"
        print(glststr)
        input()
    return


def subMenuExit():
    global reqexit
    kanban.save(storefile)
    clear_screen()
    draw_warning(title="Database",
                 text="Your personal information was saved.\n\n\
                 Come back soon. ;-)")
    input()
    clear_screen()
    reqexit = True
    return


def subMenuAdd():
    print("Insert a new optional task.", end="\n\n")
    newtask = input(" Task name: ")
    newtask = (newtask.strip())[:70]
    if not newtask.isspace() and len(newtask) >= 3:
        try:
            kanban.addOption(newtask)
        except Exception as ex:
            draw_warning(title="Exception", text=str(ex))
            input()
    else:
        draw_warning(title="Notice", text="Action cancelled.\n\
                     Nothing was added.")
        input()
    return


def promoteElement(elementType: int, elementName: str):
    if(elementType == 1):
        try:
            kanban.promoteOption(elementName)
        except Exception as ex:
            draw_warning(title="Exception", text=str(ex))
            input()
    elif(elementType == 2):
        try:
            kanban.promoteTodo(elementName)
        except Exception as ex:
            draw_warning(title="Exception", text=str(ex))
            input()
    return


def subMenuPromote():
    print("✪ Promote what? ✪", end="\n\n")

    submenulist = []

    if(len(kanban.tasks["options"]) > 0):
        submenulist.insert(0, "Option")
    if(len(kanban.tasks["todo"]) > 0):
        submenulist.insert(0, "Todo")

    submenu = Termenu(submenulist, multiselect=False)

    qresult = submenu.show()
    if(qresult == "Option"):
        optmenu = Termenu(kanban.tasks["options"], multiselect=False)

        qresult = optmenu.show()
        if qresult is not None:
            promoteElement(elementType=1, elementName=qresult)
        del optmenu
    elif(qresult == "Todo"):
        todomenu = Termenu(kanban.tasks["todo"], multiselect=False)

        qresult = todomenu.show()
        if qresult is not None:
            promoteElement(elementType=2, elementName=qresult)
        del todomenu
    return


def penalizeElement(elementType: int, elementName: str):
    if(elementType == 1):
        try:  # Le opzioni scompaiono per sempre dopo la rimozione
            kanban.removeOption(elementName)
        except Exception as ex:
            draw_warning(title="Exception", text=str(ex))
            input()
    elif(elementType == 2):
        try:
            kanban.penalizeTodo(elementName)
        except Exception as ex:
            draw_warning(title="Exception", text=str(ex))
            input()
    return


def subMenuPenalize():
    print("❰ Penalize what? ❱", end="\n\n")

    submenulist = []
    if(len(kanban.tasks["options"]) > 0):
        submenulist.insert(0, "Option")
    if(len(kanban.tasks["todo"]) > 0):
        submenulist.insert(0, "Todo")

    submenu = Termenu(submenulist, multiselect=False)

    qresult = submenu.show()
    if(qresult == "Option"):
        optmenu = Termenu(kanban.tasks["options"], multiselect=False)

        qresult = optmenu.show()
        if qresult is not None:
            penalizeElement(elementType=1, elementName=qresult)
        del optmenu
    elif(qresult == "Todo"):
        todomenu = Termenu(kanban.tasks["todo"], multiselect=False)

        qresult = todomenu.show()
        if qresult is not None:
            penalizeElement(elementType=2, elementName=qresult)
        del todomenu
    return


def subMenuClear():
    wtext = """This will delete: todo, options and completed tasks.
The database will be replaced PERMANENTLY with a clear space.\n
Press ENTER on Back in the next menu to cancel this action.
"""
    draw_warning(title="Warning - READ CAREFULLY", text=wtext)
    input()
    ansi.clear_screen()
    ansi.restore_position()
    print("✘ Clear the database... ✘")
    print("This will delete: todo, options and completed tasks.",
          end="\n\n")
    killerentry = ansi.highlight("Clear the database", "red")
    resmenu = Termenu(["Back", killerentry, "Back"], multiselect=False)

    qresult = resmenu.show()
    del resmenu
    if(qresult != "Back"):
        print("Deleting the data.")
        print("Press CTRL+C at any time to abort.")
        time.sleep(1.0)
        ansi.save_position()
        for remt in range(3, 0, -1):
            print(remt, end='')
            ansi.restore_position()
            sys.stdout.flush()
            time.sleep(1.0)
            ansi.clear_line()
            sys.stdout.flush()
        kanban.clear()
        draw_warning(title="Database", text="Personal database cleared.")
    else:
        draw_warning(title="Database",
                     text="Action cancelled.\nNothing done.")
    input()


def standardOperation():
    global reqexit
    os.system("clear")
    clear_screen()
    menulist = ["List", "Add", "EXIT"]

    totalelements = len(kanban.tasks["options"]) + len(kanban.tasks["todo"])
    if(totalelements > 0):
        menulist.insert(2, "Promote")
        menulist.insert(3, "Penalize")
        menulist.insert(-1, "Clear")

    menu = Termenu(menulist, multiselect=False)
    menu.scroll = False

    print("Personal Kanban", end="\n\n")
    ansi.save_position()
    print("")
    draw_kanban()

    qresult = menu.show()
    clear_screen()
    del menu
    if(qresult == "List"):
        subMenuList()
    elif(qresult == "EXIT"):
        subMenuExit()
    elif(qresult == "Add"):
        subMenuAdd()
    elif(qresult == "Promote"):
        subMenuPromote()
    elif(qresult == "Penalize"):
        subMenuPenalize()
    elif(qresult == "Clear"):
        subMenuClear()


while(reqexit is False):
    standardOperation()

ansi.move_horizontal(1)
